% make Figure 10
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Markovian Dynamics on Reaction Networks       %%%%%%%%%%%%
%%%%%%%%%%%             Example: Neural dynamics               %%%%%%%%%%%%
%%%%%%%%%%%                AVALANCHE ANALYSIS                  %%%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias             %%%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 04/24/2012             %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tf           = 1000;    % simulation time (in msec)
timeStep     = .5;      % time step (in msec)
monteSamps   = 1000;    % number of Monte Carlo samples
numParams    = 10;      % number of parameter combinations

L            = 100;     % total number of neurons (even number)
h            = 0.001;   % external neuronal input
gamma        = 0.1;     % neuron decay rate 1/msec

nu_E_plus_nu_I = 0.004;

nu_E_minus_nu_I = linspace(0.004,0.076,numParams); % the x axis of Fig. 10

%
% Initialize vectors and compute entropy, entropy production rate 
% and number of avalanches at each set of parameter values
%

S       = zeros(numParams,1);
entProd = zeros(numParams,1);
numAval = zeros(numParams,1);

for index = 1:numParams
    
    nu_E = .5*(nu_E_plus_nu_I+nu_E_minus_nu_I(index));
    nu_I = .5*(nu_E_plus_nu_I-nu_E_minus_nu_I(index));
    
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Compute thermodynamic quantities
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %

    [S(index),entProd(index)] = neural(h,nu_E,nu_I,gamma,tf,timeStep,L);

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Compute avalanches and avalancehe formation rate
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    
    numAval(index) = aveaval(h,nu_E,nu_I,gamma,tf,L,monteSamps);
    rateAval = numAval/tf;
    
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make and save plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

lnw = 2;

figure;
fig1 = plot(nu_E_minus_nu_I,rateAval,'--ro','LineWidth',lnw,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','r',...
                'MarkerSize',10);
axis([0 0.08 0 0.025]);
xlabel('\delta\nu = \nu_E - \nu_I');
ylabel('average avalanche formation rate');

figure;
fig2 = plot(nu_E_minus_nu_I,S,'--bo','LineWidth',lnw,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','b',...
                'MarkerSize',10);
axis([0 0.08 4 6]);
xlabel('\delta\nu = \nu_E - \nu_I');
ylabel('entropy at steady-state');

figure;
fig3 = plot(nu_E_minus_nu_I,entProd,'--go','LineWidth',lnw,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',10);
axis([0 0.08 0 1.4]);
xlabel('\delta\nu = \nu_E - \nu_I');
ylabel('supplied motive power at steady-state');

saveas(fig1,'results/rateAval.fig');
saveas(fig2,'results/entropy.fig');
saveas(fig3,'results/motPower.fig');