%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%               OPINION FORMATION                    %%%%%%%%%%%%
%%%%%%%%%%%           MOMENT APPROXIMATION METHOD              %%%%%%%%%%%%
%%%%%%%%%%%           Authors: Jenkinson/Goutsias              %%%%%%%%%%%%
%%%%%%%%%%%            Last Modified: 04/18/2012               %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Compute the first and second order derivatives of the propensities
%
function [ alpha, h_m1_m2, h_m_m1_m2 ] = compHessGrad(mu,k1,k2,a1,a2,a3,L)

%
% initialize variables
%
alpha     = zeros(4,1); 
h_m1_m2   = zeros(4,4);
h_m_m1_m2 = zeros(4,4,4);

%
% Compute propensities
%

alpha(1) = k1*( L-mu(1)+mu(2) )  * exp( a1*(mu(1)-mu(2))+a2*(mu(3)-mu(4)));
alpha(2) = k1*( L+mu(1)-mu(2) )  * exp(-a1*(mu(1)-mu(2))-a2*(mu(3)-mu(4)));
alpha(3) = k1*( L-mu(3)+mu(4) )  * exp(  a3*(mu(1)-mu(2)) );
alpha(4) = k1*( L+mu(3)-mu(4) )  * exp( -a3*(mu(1)-mu(2)) );

%
% Compute gradients
%

h_m1_m2(1,1) = k1*( a1*L-a1*mu(1)+a1*mu(2)-1 ) * exp( a1*(mu(1)-mu(2))...
                   + a2*(mu(3)-mu(4)) );
h_m1_m2(2,1) = k1*(-a1*L-a1*mu(1)+a1*mu(2)+1 ) * exp( -a1*(mu(1)-mu(2))...
                   - a2*(mu(3)-mu(4)) );
h_m1_m2(3,1) =  a3*k2*( L - mu(3) + mu(4) )    * exp( a3*(mu(1)-mu(2)) );
h_m1_m2(4,1) = -a3*k2*( L + mu(3) - mu(4) )    * exp( -a3*(mu(1)-mu(2)) );

h_m1_m2(1,2) = k1*(-a1*L+a1*mu(1)-a1*mu(2)+1 ) * exp( a1*(mu(1)-mu(2))...
                   + a2*(mu(3)-mu(4)) );
h_m1_m2(2,2) = k1*( a1*L+a1*mu(1)-a1*mu(2)-1 ) * exp( -a1*(mu(1)-mu(2))...
                   - a2*(mu(3)-mu(4)) );
h_m1_m2(3,2) = -a3*k2*( L - mu(3) + mu(4) )    * exp( a3*(mu(1)-mu(2)) );
h_m1_m2(4,2) =  a3*k2*( L + mu(3) - mu(4) )    * exp( -a3*(mu(1)-mu(2)) );

h_m1_m2(1,3) =  a2*k1*( L-mu(1)+mu(2) ) * exp(  a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m1_m2(2,3) = -a2*k1*( L+mu(1)-mu(2) ) * exp( -a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m1_m2(3,3) = -k2                      * exp( a3*(mu(1)-mu(2)) );
h_m1_m2(4,3) =  k2                      * exp( -a3*(mu(1)-mu(2)) );

h_m1_m2(1,4) = -a2*k1*( L-mu(1)+mu(2) ) * exp(  a1*(mu(1)-mu(2))...
               + a2*(mu(3)-mu(4)) );
h_m1_m2(2,4) =  a2*k1*( L+mu(1)-mu(2) ) * exp( -a1*(mu(1)-mu(2))...
                - a2*(mu(3)-mu(4)) );
h_m1_m2(3,4) =  k2                      * exp( a3*(mu(1)-mu(2)) );
h_m1_m2(4,4) = -k2                      * exp( -a3*(mu(1)-mu(2)) );

%
% Compute Hessians of alpha_1
%

h_m_m1_m2(1,1,1) = k1*( a1^2*L-a1^2*mu(1)+a1^2*mu(2)-2*a1 ) * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,1,2) = k1*(-a1^2*L+a1^2*mu(1)-a1^2*mu(2)-1 )    * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,1,3) =  a2*k1*( a1*L-a1*mu(1)+a1*mu(2)-1 )      * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,1,4) = -a2*k1*( a1*L-a1*mu(1)+a1*mu(2)-1 )      * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );

h_m_m1_m2(1,2,1) = k1*(-a1^2*L+a1^2*mu(1)-a1^2*mu(2)+2*a1) * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,2,2) = k1*( a1^2*L-a1^2*mu(1)+a1^2*mu(2)-2*a1) * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,2,3) =  a2*k1*(-a1*L+a1*mu(1)-a1*mu(2)+1)      * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,2,4) = -a2*k1*(-a1*L+a1*mu(1)-a1*mu(2)+1)      * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );

h_m_m1_m2(1,3,1) = a2*k1*( a1*L-a1*mu(1)+a1*mu(2)-1)      * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,3,2) = a2*k1*(-a1*L+a1*mu(1)-a1*mu(2)+1)      * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,3,3) =  a2^2*k1*(L-mu(1)+mu(2))               * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,3,4) = -a2^2*k1*(L-mu(1)+mu(2))               * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );

h_m_m1_m2(1,4,1) = -a2*k1*( a1*L-a1*mu(1)+a1*mu(2)-1)     * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,4,2) = -a2*k1*(-a1*L+a1*mu(1)-a1*mu(2)+1)     * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,4,3) = -a2^2*k1*(L-mu(1)+mu(2))               * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );
h_m_m1_m2(1,4,4) =  a2^2*k1*(L-mu(1)+mu(2))               * exp( a1*(mu(1)-mu(2)) + a2*(mu(3)-mu(4)) );

%
% Compute Hessians of alpha_2
%

h_m_m1_m2(2,1,1) = k1*( a1^2*L+a1^2*mu(1)-a1^2*mu(2)-2*a1)  * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,1,2) = k1*(-a1^2*L-a1^2*mu(1)+a1^2*mu(2)+2*a1)  * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,1,3) = -a2*k1*(-a1*L-a1*mu(1)+a1*mu(2)+1)       * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,1,4) =  a2*k1*(-a1*L-a1*mu(1)+a1*mu(2)+1)       * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );

h_m_m1_m2(2,2,1) = k1*(-a1^2*L-a1^2*mu(1)+a1^2*mu(2)+2*a1) * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,2,2) = k1*( a1^2*L+a1^2*mu(1)-a1^2*mu(2)-2*a1) * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,2,3) = -a2*k1*( a1*L+a1*mu(1)-a1*mu(2)-1)      * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,2,4) =  a2*k1*( a1*L+a1*mu(1)-a1*mu(2)-1)      * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );

h_m_m1_m2(2,3,1) = -a2*k1*(-a1*L-a1*mu(1)+a1*mu(2)+1)      * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,3,2) = -a2*k1*( a1*L+a1*mu(1)-a1*mu(2)-1)      * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,3,3) =  a2^2*k1*( L+mu(1)-mu(2))               * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,3,4) = -a2^2*k1*( L+mu(1)-mu(2))               * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );

h_m_m1_m2(2,4,1) = a2*k1*(-a1*L-a1*mu(1)+a1*mu(2)+1)       * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,4,2) = a2*k1*( a1*L+a1*mu(1)-a1*mu(2)-1)       * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,4,3) = -a2^2*k1*( L+mu(1)-mu(2))               * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );
h_m_m1_m2(2,4,4) =  a2^2*k1*( L+mu(1)-mu(2))               * exp(-a1*(mu(1)-mu(2)) - a2*(mu(3)-mu(4)) );

%
% Compute Hessians of alpha_3
%

h_m_m1_m2(3,1,1) =  a3^2*k2*(L-mu(3)+mu(4)) * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,1,2) = -a3^2*k2*(L-mu(3)+mu(4)) * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,1,3) = -a3*k2                   * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,1,4) =  a3*k2                   * exp( a3*(mu(1)-mu(2)) );

h_m_m1_m2(3,2,1) = -a3^2*k2*(L-mu(3)+mu(4)) * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,2,2) =  a3^2*k2*(L-mu(3)+mu(4)) * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,2,3) =  a3*k2                   * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,2,4) = -a3*k2                   * exp( a3*(mu(1)-mu(2)) );

h_m_m1_m2(3,3,1) = -a3*k2                   * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,3,2) =  a3*k2                   * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,3,3) =  0;
h_m_m1_m2(3,3,4) =  0;

h_m_m1_m2(3,4,1) =  a3*k2                   * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,4,2) = -a3*k2                   * exp( a3*(mu(1)-mu(2)) );
h_m_m1_m2(3,4,3) =  0;
h_m_m1_m2(3,4,4) =  0;

%
% Compute Hessians of alpha_4
%

h_m_m1_m2(4,1,1) =  a3^2*k2*(L+mu(3)-mu(4)) * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,1,2) = -a3^2*k2*(L+mu(3)-mu(4)) * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,1,3) = -a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,1,4) =  a3*k2                   * exp( -a3*(mu(1)-mu(2)) );

h_m_m1_m2(4,2,1) = -a3^2*k2*(L+mu(3)-mu(4)) * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,2,2) =  a3^2*k2*(L+mu(3)-mu(4)) * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,2,3) =  a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,2,4) = -a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
 
h_m_m1_m2(4,3,1) = -a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,3,2) =  a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,3,3) =  0;
h_m_m1_m2(4,3,4) =  0;

h_m_m1_m2(4,4,1) =  a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,4,2) = -a3*k2                   * exp( -a3*(mu(1)-mu(2)) );
h_m_m1_m2(4,4,3) =  0;
h_m_m1_m2(4,4,4) =  0;